describe Orator::EventHandler do
  it "should initalize with block" do
    expect {
      EventHandler.new { raise StandardError }
    }.to raise_error(StandardError)
  end

  it "should add an event with a block" do
    subject.events.length.should be 0
    subject.on(:some_event) { something }
    subject.events.length.should be 1
    subject.events.first[:event].should eq "some_event"
    subject.events.first[:block].should be_kind_of Proc
  end

  class SomeClass < Orator::Base; end

  it "should accept a class for an event" do
    subject.on(:some_event, SomeClass)
    subject.events.first[:class].should be SomeClass
  end

  it "should accept a count" do
    subject.on(:some_event, SomeClass, 4)
    subject.events.first[:count].should be 4
  end

  it "should decrease the count by one" do
    something = double("something")
    something.should_receive(:to_call)
    subject.on(:some_event, nil, 4) { something.to_call }
    subject.events.first[:count].should be 4
    subject.trigger(:some_event, self)
    subject.events.first[:count].should be 3
  end

  it "shouldn't add the same event twice" do
    subject.on(:some_event, SomeClass)
    subject.events.length.should be 1
    subject.on(:some_event, SomeClass)
    subject.events.length.should be 1
  end

  it "should remove events with a count of 0" do
    something = double("something")
    something.should_receive(:to_call)
    subject.on(:some_event, nil, 1) { something.to_call }
    subject.on(:another_event) { }
    subject.events.length.should be 2
    subject.trigger(:some_event, self)
    subject.events.length.should be 1
  end

  it "should select events with the same event" do
    subject.on(:some_event) { }
    subject.on(:another_event) { }
    subject.on(:some_event) { do_something }

    subject.events.length.should be 3
    subject.events(:some_event).length.should be 2
  end

  it "should call socket.missing on an empty event" do
    something = double('something')
    something.should_receive(:to_call)
    subject.on('socket.missing') { something.to_call }
    subject.trigger('some.event', self)
  end

  it "should not call socket.missing on an event" do
    something = double('something')
    something.should_not_receive(:to_call)
    something.should_receive(:should_call)
    subject.on('socket.missing') { something.to_call }
    subject.on('some.event') { something.should_call }
    subject.trigger('some.event', self)
  end
end
